#include <iostream>
#include <chrono>
#include <thread>
#include <mutex>
#include <condition_variable>

#define PERIOD_COUNTER 300


void set_raw(bool set) {
    if (set) {
        system("stty raw");  // enable raw
    } else {
        system("stty -raw"); // disable raw
    }
}

class tic_tac_toe {
public:
    explicit tic_tac_toe() : quit(false), counter(0) {};
    void input() {
        std::unique_lock<std::mutex> lg(mutex);
        lg.unlock();
        char c;
        std::cin >> c;
        if (c == 'q') {
            lg.lock();
            set_quit(true);
            cvar.notify_all();
        }
    }
    void output_area() {
        std::unique_lock<std::mutex> lg(mutex);
        cvar.wait(lg);
        if (!get_quit())
            std::cout << "\rCounter: " << counter << std::endl;
    }
    void take() {
        std::this_thread::sleep_for(std::chrono::milliseconds(PERIOD_COUNTER));
        std::unique_lock<std::mutex> lg(mutex);
        ++counter;
        cvar.notify_one();
    }
    bool get_quit() const {
        return quit;
    }
    void set_quit(const bool qq) {
        quit = qq;
    }
private:
    bool quit;
    int counter;
    std::mutex mutex;
    std:: condition_variable cvar;
};

int main() {
    auto inputThread = [](tic_tac_toe& player) {
        std::cout << "\rStart input thread\n";
        bool q = false;
        while (!q) {
            player.input();
            q = player.get_quit();
        }
        std::cout << "\rExit input thread\n";
    };
    auto computeThread = [](tic_tac_toe& player) {
        std::cout << "\rStart compute thread\n";
        bool q = false;
        while (!q) {
            player.take();
            q = player.get_quit();
        }
        std::cout << "\rExit compute thread\n";
    };
    auto outputThread = [](tic_tac_toe& player) {
        std::cout << "\rStart output thread\n";
        bool q = false;
        while (!q) {
            player.output_area();
            q = player.get_quit();
        }
        std::cout << "\rExit output thread\n";
    };

    set_raw(true);

    tic_tac_toe PP;
    std::thread t1(inputThread, std::ref(PP));
    std::thread t2(computeThread, std::ref(PP));
    std::thread t3(outputThread, std::ref(PP));

    t1.join();
    t2.join();
    t3.join();

    set_raw(false);

    std::cout << "\rExit main thread\n";
    return 0;
}
